/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2014
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev$
 * $Date: 2012/02/28 13:28:18 $
 * $Author: posch $
 *
 */
package de.unihalle.informatik.Alida.demo;

import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.annotations.ALDAOperator;
import de.unihalle.informatik.Alida.annotations.Parameter;

/**
 * Operator for calculating the mean of given double values.
 * 
 * @author moeller
 */
@ALDAOperator(genericExecutionMode=ALDAOperator.ExecutionMode.ALL)
public class ALDCalcMean extends ALDOperator {

  /**
   * Input data array.
   */
  @Parameter( label= "Input data", required = true, 
  		direction = Parameter.Direction.IN, 
  		description = "Input data.")
  private Double [] data;

  /**
   * Compute mean free data
   */
  @Parameter( label= "Compute mean free data", required = false, 
  		direction = Parameter.Direction.INOUT, 
  		description = "Optionally compute mean free data.")
  private Boolean doMeanFree = new Boolean(false);

  /**
   * Calculated average value.
   */
  @Parameter( label= "Mean value",  
  		direction = Parameter.Direction.OUT, 
  		description = "Result.")
  private transient Double mean = null;

  /**
   * Mean free data
   */
  @Parameter( label= "Mean free data",  
  		direction = Parameter.Direction.OUT, 
  		description = "Result.")
  private transient Double[] meanFreeData = null;

	/**
	 * Default constructor.
	 * @throws ALDOperatorException
	 */
	public ALDCalcMean() throws ALDOperatorException {
		super();
	}

	/**
	 * Constructor.
	 * 
	 * @param _data	Input data.
	 * @throws ALDOperatorException
	 */
	public ALDCalcMean(Double [] _data) throws ALDOperatorException {
		super();
		this.data = _data;
	}

	@Override
	protected void operate() {
		// calculate the mean of the data
		double tmpMean = 0.0;
		for (int i=0; i<this.data.length; ++i) {
			tmpMean += this.data[i].doubleValue();
		}
		this.mean = new Double(tmpMean/this.data.length);

		if ( this.doMeanFree.booleanValue() ) {
			this.meanFreeData = new Double[this.data.length];
			for (int i=0; i<this.data.length; ++i) {
				this.meanFreeData[i] = 
						new Double(this.data[i].doubleValue() - this.mean.doubleValue());
			}
		}
	}
	
	/**
	 * Returns the calculated mean value.
	 */
	public Double getResultMean() {
		return this.mean;
	}
}
