/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2014
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.gui;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.help.BadIDException;
import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.help.HelpSetException;


/**
 * Convenience class for context-sensitive online help. 
 * 
 * @author moeller
 */
public class OnlineHelpDisplayer {

	/**
	 * Name of helpset to be used.
	 */
	private static String helpHS = "main_en_US.hs";

	/**
	 * Local help broker.
	 */
	static HelpBroker hb = null;
	
	// static initialization routine to load helpset only once
	static {
    try {
    	ClassLoader cl = OnlineHelpDisplayer.class.getClassLoader();
  		URL hsURL = HelpSet.findHelpSet(cl, helpHS);
  		HelpSet hs = new HelpSet(null, hsURL);
			hb = hs.createHelpBroker();
    } catch (HelpSetException e) {
    	System.err.println("HelpDisplay: static init failed!!!");
    	e.printStackTrace();
    }
	}			
	
	/**
	 * Get an ActionListener linked to the Alida online help system.
	 * 
	 * @param c	Component to which to attach the listener (buttons, menu items).
	 * @param l	Label of the page in the helpset to be displayed.
	 * @return	Listener for the component.
	 */
	public static ActionListener getHelpActionListener(Component c,String l) {
		// make sure that the page can be displayed
		String s = l;
		try {
			OnlineHelpDisplayer.hb.setCurrentID(s);
		} catch(BadIDException ex) {
//			System.err.println("Help index " + s + " does not exist!!");
//			System.err.println("-> switching back to main page...");
			// switch back to welcome page if ID not found
			s = "welcome";
		}
		CSH.setHelpIDString(c, s);
		// generate the ActionListener
    javax.help.CSH.DisplayHelpFromFocus helpdisp = 
			new CSH.DisplayHelpFromFocus(hb);
    return helpdisp;
  }
}
